local addon = LibStub("AceAddon-3.0"):GetAddon("Langoustine")
local tanks = addon:NewModule("Tanks", "AceEvent-3.0","AceConsole-3.0")
local LSM = LibStub:GetLibrary("LibSharedMedia-3.0")

local TankHeader,TankAnchor = nil, nil
ClickCastFrames = ClickCastFrames or {}

local function ConfigureFunction(uf)
	local db = addon.db.profile.TankOptions
	print(uf:GetAttribute("unit"))
	uf:SetAttribute("*type1", "target")
	-- we switch the tank unit it in a vechicle
	uf:SetAttribute("toggleForVehicle",true)
	uf:RegisterForClicks("AnyUp")
	uf.isMT = true
	--uf:SetScript("OnAttributeChanged", nil) -- TODO add calback to refresh unit information if it has switched!
	tanks:SetupFrame(uf)
	tanks:DecorateFrame(uf)
	-- Setup target
	uf.target = CreateFrame("Button",uf:GetName().."Target",uf,"SecureActionButtonTemplate")
	uf.target:SetAttribute("useparent-unit", true)
	uf.target:SetAttribute("unitsuffix", "target")
	uf.target:SetAttribute("*type1","target")
	uf.target:RegisterForClicks("AnyUp")	
	tanks:SetupFrame(uf.target)
	tanks:DecorateFrame(uf.target)
	if not addon.db.profile.TankShowTarget then
		uf.target:Hide()
	end
	uf.target:SetPoint("RIGHT",uf,"RIGHT",db.unitWidth+3,0)
	-- setup ToT
	uf.tot = CreateFrame("Button",uf:GetName().."TargetTarget",uf,"SecureActionButtonTemplate")
	uf.tot:SetAttribute("useparent-unit", true)
	uf.tot:SetAttribute("unitsuffix", "targettarget")
	uf.tot:SetAttribute("*type1","target")
	uf.tot:RegisterForClicks("AnyUp")
	tanks:SetupFrame(uf.tot)
	tanks:DecorateFrame(uf.tot)
	uf.tot:SetPoint("RIGHT",uf.target,"RIGHT",db.unitWidth+3,0)	
	if not addon.db.profile.TankShowTargetTarget then
		uf.tot:Hide()
	end
	-- Setup ClickCastFrames
	if db.enableClickCast then
		ClickCastFrames[uf] = true
		ClickCastFrames[uf.target] = true
		ClickCastFrames[uf.tot] = true
	end
	tinsert(tanks.tankFrames, uf)
end

function tanks:OnInitialize()
	self.tankFrames = {}
	self.localTanks = {}
	self:SetupUI()
	self.timer = 0
end
function tanks:ToggleAnchorLock()
	local db = addon.db.profile.TankOptions
	if db.locked then
		TankAnchor:SetMovable(false)
		TankAnchor:RegisterForDrag(nil)
	else
		TankAnchor:SetMovable(true)
		TankAnchor:RegisterForDrag("LeftButton")
	end
end

function tanks:SetupUI()
	local db = addon.db.profile.TankOptions
	TankAnchor = CreateFrame("Frame","LTA",UIParent)
	TankAnchor:SetWidth(db.unitWidth)
	TankAnchor:SetHeight(15)
	TankAnchor:SetPoint("CENTER",UIParent,"CENTER",db.x,db.y)
	TankAnchor.label = TankAnchor:CreateFontString(nil,"OVERLAY","GameFontWhite")
	TankAnchor.label:SetAllPoints(TankAnchor)
	TankAnchor.label:SetText("Tank Anchor")
	TankAnchor.label:Show()
	TankAnchor:SetBackdrop(
		{
		--bgFile = [[Interface/Tooltips/UI-Tooltip-Background]], 
	 	--edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], 
	 	--tile = false, tileSize = 16, edgeSize = 8, 
	 	insets = { left = 2, right = 2, top = 2, bottom = 2 }})
	
	TankAnchor:SetBackdropColor(0,0,0,1)
	TankAnchor:EnableMouse(true)
	TankAnchor:SetMovable(true)
	TankAnchor:RegisterForDrag("LeftButton")
	TankAnchor:SetScript("OnDragStart", function(self) if not InCombatLockdown() then self:StartMoving() end end)
	TankAnchor:SetScript("OnDragStop", function(self) 
		if not InCombatLockdown() then 
			self:StopMovingOrSizing() 
			local scale,pscale = self:GetEffectiveScale(),self:GetParent():GetEffectiveScale()
			local to,anchor,from,x,y = self:GetPoint()
			local gX,gY = self:GetLeft() + self:GetWidth() / 2, self:GetBottom() + self:GetHeight() / 2
			local pX,pY = UIParent:GetLeft() + UIParent:GetWidth() / 2, UIParent:GetBottom() + UIParent:GetHeight() / 2
			local x = (gX * scale) - (pX * pscale)
			local y = (gY * scale) - (pY * pscale)
			x = x/scale
			y = y/scale
			self:ClearAllPoints()
			self:SetPoint("CENTER",UIParent,"CENTER",x,y)
			db.x = x
			db.y = y
		end 
	end)
	TankAnchor:Show()
	if db.locked then
		TankAnchor:SetMovable(false)
		TankAnchor:RegisterForDrag(nil)
	end
	----  ADD A RIGHT/LEFT CLICK to the ANCHOR to Set/Clear MT, also ned a mouseover option to show this info
	---- LDB doesnt unfortunetly use SecureButons to provide thse functions. PLUS a locking function
	
	
	TankHeader = CreateFrame("Frame","LTH",TankAnchor,"SecureRaidGroupHeaderTemplate")
	TankHeader.initialConfigFunction = ConfigureFunction
	TankHeader:SetAttribute("template","SecureUnitButtonTemplate")
	TankHeader:SetWidth(db.unitWidth)
	TankHeader:SetHeight(15)
	TankHeader:SetAttribute("minheight",15)
	-- configure sorting options here
	TankHeader:SetAttribute("sortMethod","INDEX")
	TankHeader:SetAttribute("sortDir","DESC")
	
	-- this can get complicated
	
	TankHeader:SetAttribute("unitsPerColumn",db.MaxTanks)
	TankHeader:SetAttribute("columnSpacing",2)
	self:SetGrowthDirection(addon.db.profile.TankOptions.growth)
end

function tanks:SetGrowthDirection(direction)
	TankHeader:ClearAllPoints()
	if direction == "Down" then
		TankHeader:SetAttribute("point", "BOTTOM")
		TankHeader:SetPoint("TOP", TankAnchor,"BOTTOM", 0, 0)	
	else
		TankHeader:SetAttribute("point", "TOP")
		TankHeader:SetPoint("BOTTOM", TankAnchor,"TOP", 0, 0)	
	end
end

function tanks:OnEnable()
	TankHeader:Show()
	self:RegisterEvent("RAID_ROSTER_UPDATE","RosterUpdate")
	self:RosterUpdate()
	self:RegisterChatCommand("pt","AddTank")
	self:RegisterChatCommand("ptoff","RemoveTank")
	self:OnUpdate(3)
	TankHeader:SetScript("OnUpdate",function(frame,elapsed) tanks:OnUpdate(elapsed) end)
	if addon.db.profile.TankOptions.hideAnchor then
		TankAnchor.label:Hide()
	else
		TankAnchor.label:Show()	
	end
end

function tanks:OnDisable()
	self:UnregisterEvent("RAID_ROSTER_UPDATE")
end
local healthFormat="%d%%"
function tanks:UpdateTankFrame(frame)
	if not frame then return end
	local unit = SecureButton_GetUnit(frame)
	if unit and UnitExists(unit) then
		local name = UnitName(unit)
		-- Unit dies .. pour the water!
		frame.name:SetText(name)
		local cur, max = UnitHealth(unit) or 0, UnitHealthMax(unit) or 0
		local perc = ceil((cur / max) * 100)
		frame.health:SetText(format(healthFormat,perc))
		frame.bar:SetValue(perc)
		local icon = GetRaidTargetIndex(unit)
		if icon and db.icon then
			SetRaidTargetIconTexture(frame.icon, icon)
			frame.icon:Show()
		else
			frame.icon:Hide()
		end
		frame:Show()
		-- Highlighting and color controls
	else
		frame:Hide()
	end
end

function tanks:RestyleWindows()
	for index,frame in pairs(tanks.tankFrames) do
		self:DecorateFrame(frame)
		self:DecorateFrame(frame.target)
		self:DecorateFrame(frame.tot)
	end
end

function tanks:OnUpdate(elapsed)
	self.timer = self.timer + elapsed
	if self.timer > 0.3 then -- we update tank frames every 1/3rd of a sec
		for index,frame in pairs(tanks.tankFrames) do
			frame:Hide()
			if addon.db.profile.TankShow then
				self:UpdateTankFrame(frame)
			end
			if addon.db.profile.TankShowTarget then
				self:UpdateTankFrame(frame.target)
			end
			if addon.db.profile.TankShowTargetTarget then
				self:UpdateTankFrame(frame.tot)
			end
		end
		self.timer = 0
	end
end

function tanks:RosterUpdate()
	local raiders = GetNumRaidMembers()
	if raiders == 0 then return end
	local tankCount = 0
	local nameList = ""
	for i=1,raiders,1 do
		local name, _, _, _, _, _, _, _, _, role, _ = GetRaidRosterInfo(i);
		if role == "MAINTANK" then
			nameList = name..","..nameList
			tankCount = tankCount + 1
		end
	end
	for k,v in pairs(self.localTanks) do
		nameList = k..","..nameList
		tankCount = tankCount + 1
	end
	-- add PT namelist here
	TankHeader:SetAttribute("nameList",nameList)
end

function tanks:DecorateFrame(f)
	local db = addon.db.profile.TankOptions
	f:SetWidth(db.unitWidth)
	f:SetHeight(db.unitHeight)
	f:SetScale(db.scale)
	f:SetAlpha(db.alpha)
	
	f.bar:SetStatusBarTexture(LSM:Fetch("statusbar",db.barTexture,true))
	f.bar:SetWidth(db.unitWidth-6)
	f.bar:SetHeight(db.unitHeight-4)
	f.bar:SetPoint("LEFT",f,"LEFT",4,0)
	f.bar:SetStatusBarColor(1,0,0)
	
	f.bg:SetWidth(db.unitWidth)
	f.bg:SetHeight(db.unitHeight)
	f.bg:SetTexture(0,0,0,0.3)
	f.bg:SetPoint("LEFT",f,"LEFT",0,0)


	f.health:SetJustifyH("RIGHT")
	f.health:SetPoint("RIGHT",f,"RIGHT",-2,0)

	f.name:SetJustifyH("LEFT")
	f.name:SetPoint("LEFT",f,"LEFT",16,0)
	
	f.icon:SetPoint("LEFT",f,"LEFT",4,0)
	f.icon:SetWidth(db.unitHeight)
	f.icon:SetHeight(db.unitHeight)
	
	if db.enableClickCast then
		ClickCastFrames[f] = true
	else
		ClickCastFrames[f] = nil
	end
	
	self:UpdateTankFrame(f)
end

function tanks:SetupFrame(f)
	local db = addon.db.profile.TankOptions
	f:SetBackdrop({
		{bgFile = [[Interface/Tooltips/UI-Tooltip-Background]], 
		 edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]], 
		 edgeSize = 8,
         insets = { left = 2, right = 2, top = 2, bottom = 2 }}
	})
	f:SetBackdropColor(0,1,0,1)
	f:SetBackdropBorderColor(0,1,0)
	-- Tank Statusbar
	f.bar = CreateFrame("StatusBar", nil, f)
	f.bar:SetMinMaxValues(0,100)

	f.bg = f.bar:CreateTexture(nil,"BACKGROUND")
	-- Tank Health
	f.health = f.bar:CreateFontString(nil, "OVERLAY","GameFontHighlightSmall")
	-- Tank Name
	f.name = f.bar:CreateFontString(nil, "OVERLAY","GameFontHighlightSmall")
	-- Raid Icon
	f.icon = f.bar:CreateTexture(nil, "OVERLAY")
	f.icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	f.icon:Hide()
	f:Hide()
end

function tanks:AddTank(unit)
	if not unit then return end
	if not UnitInRaid(unit) then return end
	if (GetPartyAssignment("MAINTANK",unit,1)) then return end
	if self.localTanks[UnitName(unit)] then return end
	self.localTanks[UnitName(unit)]=unit
	self:RosterUpdate()
end
function tanks:RemoveTank(unit)
	if self.localTanks[UnitName(unit)] then
		self.localTanks[UnitName(unit)]=nil
	end
	self:RosterUpdate()
end